@@ -1,26 +1,34 @@
-import inspect
 import sys
+import warnings
+from getgauge.registry import registry, MessagesStore, ScreenshotsStore
 
-from getgauge.registry import registry, _MessagesStore
+if sys.version_info[0] is 3:
+    from collections.abc import MutableMapping
+else:
+    from collections import MutableMapping
 
 
 def step(step_text):
     def _step(func):
-        f_back = sys._getframe().f_back
-        registry.add_step_definition(step_text, func, f_back.f_code.co_filename, inspect.getsourcelines(func)[1])
+        f_code = sys._getframe().f_back.f_code
+        span = {'start': f_code.co_firstlineno,
+                'startChar': 0, 'end': 0, 'endChar': 0}
+        registry.add_step(step_text, func, f_code.co_filename, span)
         return func
 
     return _step
 
 
-def before_suite(func):
-    registry.add_before_suite(func)
-    return func
+def continue_on_failure(obj):
+    return _define_wrapper(obj, registry.continue_on_failure)
 
 
-def after_suite(func):
-    registry.add_after_suite(func)
-    return func
+def before_suite(obj=None):
+    return _define_wrapper(obj, registry.add_before_suite)
+
+
+def after_suite(obj=None):
+    return _define_wrapper(obj, registry.add_after_suite)
 
 
 def before_scenario(obj=None):
@@ -39,19 +47,36 @@ def after_spec(obj=None):
     return _define_wrapper(obj, registry.add_after_spec)
 
 
-def before_step(obj):
+def before_step(obj=None):
     return _define_wrapper(obj, registry.add_before_step)
 
 
-def after_step(obj):
+def after_step(obj=None):
     return _define_wrapper(obj, registry.add_after_step)
 
 
 def screenshot(func):
-    registry.set_screenshot_provider(func)
+    _warn_screenshot_deprecation('screenshot', 'custom_screenshot_writer')
+    registry.set_screenshot_provider(func, False)
     return func
 
 
+def custom_screen_grabber(func):
+    _warn_screenshot_deprecation('custom_screen_grabber', 'custom_screenshot_writer')
+    registry.set_screenshot_provider(func, False)
+    return func
+
+def custom_screenshot_writer(func):
+    registry.set_screenshot_provider(func, True)
+    return func
+
+def _warn_screenshot_deprecation(old_function, new_function):
+    warnings.warn(
+        "'{0}' is deprecated in favour of '{1}'".format(old_function, new_function),
+        DeprecationWarning, stacklevel=3)
+    warnings.simplefilter('default', DeprecationWarning)
+
+
 class Table:
     def __init__(self, proto_table):
         self.__headers = proto_table.headers.cells
@@ -76,7 +101,16 @@ class Table:
         return [row.cells[index - 1] for row in self.__rows]
 
     def __str__(self):
-        return '{}\n{}'.format(', '.join(self.__headers), '\n'.join([', '.join(x.cells) for x in self.__rows]))
+        table = [""] * (len(self.__rows) + 2)
+        for header in self.__headers:
+            values = [header, ""] + self.get_column_values_with_name(header)
+            m_length = max(map(len, values))
+            values[1] = "-" * m_length
+            values = [value.ljust(m_length) for value in values]
+            for i, value in enumerate(values):
+                table[i] = table[i] or []
+                table[i].append(value)
+        return "\n".join(["|{}|".format("|".join(row)) for row in table])
 
     def __eq__(self, other):
         return self.__str__() == other.__str__()
@@ -139,8 +173,10 @@ class Specification:
 
     def __str__(self):
         return "Specification: {{ name: {}, is_failing: {}, tags: {}, file_name: {} }}".format(self.name,
-                                                                                               str(self.is_failing),
-                                                                                               ", ".join(self.tags),
+                                                                                               str(
+                                                                                                   self.is_failing),
+                                                                                               ", ".join(
+                                                                                                   self.tags),
                                                                                                self.file_name)
 
     def __eq__(self, other):
@@ -174,7 +210,9 @@ class Scenario:
 
 
 class Step:
-    def __init__(self, text, is_failing):
+    def __init__(self, text, is_failing, message="", stacktrace=""):
+        self.__stacktrace = stacktrace
+        self.__error_message = message
         self.__text = text
         self.__is_failing = is_failing
 
@@ -186,8 +224,17 @@ class Step:
     def is_failing(self):
         return self.__is_failing
 
+    @property
+    def error_message(self):
+        return self.__error_message
+
+    @property
+    def stacktrace(self):
+        return self.__stacktrace
+
     def __str__(self):
-        return "Step: {{ text: {}, is_failing: {} }}".format(self.text, str(self.is_failing))
+        s = "Step: {{ text: {}, is_failing: {}, error_message: {}, stacktrace: {} }}"
+        return s.format(self.text, str(self.is_failing), str(self.error_message), str(self.stacktrace))
 
     def __eq__(self, other):
         return self.__str__() == other.__str__()
@@ -196,12 +243,55 @@ class Step:
 class Messages:
     @staticmethod
     def write_message(message):
-        _MessagesStore.write_message(message)
+        MessagesStore.write_message(message)
+
+
+class DictObject(dict):
+    def __getattr__(self, name):
+        try:
+            return self[name]
+        except KeyError:
+            raise AttributeError("'{0}' object has no attribute '{1}'".format(
+                self.__class__.__name__, name))
+
+    def __setattr__(self, name, value):
+        self[name] = value
+
+    def __delattr__(self, name):
+        try:
+            del self[name]
+        except KeyError:
+            raise AttributeError("'{0}' object has no attribute '{1}'".format(
+                self.__class__.__name__, name))
+
+
+class DataStoreContainer(object):
+    def __init__(self):
+        self.__scenario = DictObject()
+        self.__spec = DictObject()
+        self.__suite = DictObject()
+
+    @property
+    def scenario(self):
+        return self.__scenario
+
+    @property
+    def spec(self):
+        return self.__spec
+
+    @property
+    def suite(self):
+        return self.__suite
+
+
+data_store = DataStoreContainer()
 
 
 class DataStore:
-    def __init__(self):
-        self.__data_store = {}
+    def __init__(self, data_store=None):
+        if data_store is None:
+            data_store = {}
+        self.__data_store = data_store
 
     def get(self, key):
         return self.__data_store[key]
@@ -213,27 +303,38 @@ class DataStore:
         return key in self.__data_store
 
     def clear(self):
-        self.__data_store = {}
+        self.__data_store.clear()
 
     def __eq__(self, other):
         return self.__data_store == other.__data_store
 
 
+def _warn_datastore_deprecation(store_type):
+    warnings.warn(
+        "'DataStoreFactory.{0}_data_store()' is deprecated in favour of 'data_store.{0}'".format(
+            store_type),
+        DeprecationWarning, stacklevel=3)
+    warnings.simplefilter('default', DeprecationWarning)
+
+
 class DataStoreFactory:
-    __scenario_data_store = DataStore()
-    __spec_data_store = DataStore()
-    __suite_data_store = DataStore()
+    __scenario_data_store = DataStore(data_store.scenario)
+    __spec_data_store = DataStore(data_store.spec)
+    __suite_data_store = DataStore(data_store.suite)
 
     @staticmethod
     def scenario_data_store():
+        _warn_datastore_deprecation("scenario")
         return DataStoreFactory.__scenario_data_store
 
     @staticmethod
     def spec_data_store():
+        _warn_datastore_deprecation("spec")
         return DataStoreFactory.__spec_data_store
 
     @staticmethod
     def suite_data_store():
+        _warn_datastore_deprecation("suite")
         return DataStoreFactory.__suite_data_store
 
 
@@ -243,7 +344,8 @@ def create_execution_context_from(current_execution_info):
                       current_execution_info.currentSpec.isFailed, current_execution_info.currentSpec.tags),
         Scenario(current_execution_info.currentScenario.name, current_execution_info.currentScenario.isFailed,
                  current_execution_info.currentScenario.tags),
-        Step(current_execution_info.currentStep.step.actualStepText, current_execution_info.currentStep.isFailed)
+        Step(current_execution_info.currentStep.step.actualStepText, current_execution_info.currentStep.isFailed,
+             current_execution_info.currentStep.errorMessage, current_execution_info.currentStep.stackTrace)
     )
 
 
@@ -261,3 +363,9 @@ def _define_wrapper(obj, callback):
         return function
 
     return func
+
+
+class Screenshots:
+    @staticmethod
+    def capture_screenshot():
+        ScreenshotsStore.capture()
